-- ----------------------------------------------------------------------------------------------- --
-- Skrypt rozszerzajcy funkcjonalno Teamw o moliwo liczenia wynikw turnieju metod Pattona --
-- Autor: mkl                                                                                      --
-- ----------------------------------------------------------------------------------------------- --
-- Wersja dla turnieju liczonego BAMami                                                            --
-- Ta wersja jest fajna, bo:                                                                       --
--  * dziaa                                                                                       --
--  * nie generuje caego mnstwa pustych protokow - cz BAMowa liczy si normalnie BAMami     --
-- Ta wersja jest niefajna, bo:                                                                    --
--  * BAMy w Teamach licz rnic +/- 10 jako nieremis, wic w protokole jest 2:0/0:2,            --
--    wic jest "po cichu" wyrwnywane w jednym worze raze z wyrwnaniami za saldo,                --
--    wic generuje cae mnstwo pyta "a panie sdzio, czemu tu jest 2:0, jak miao by 1:1"      --
-- ----------------------------------------------------------------------------------------------- --
-- Instrukcja obsugi:                                                                             --
--  * uruchomi w bazie turnieju skrypt co najmniej raz po zaoeniu turnieju,                     --
--    a przed wpisaniem pierwszego wyrwnania "waciwego" (tj. faktycznej kary/wyrwnania         --
--    z turnieju)                                                                                  --
--  * uruchomi kazdorazowo celem przeliczenia wynikw                                             --
-- Potencjalne problemy:                                                                           --
--  * wyrwnania wklepywane z Teamw mog sprawia problemy (nie s dostatecznie przetestowane)    --
--  * nie mam zielonego pojcia, czy i jak powinny obchodzi mnie wyniki niezapisowe w rozdaniach  --
-- Szczeglne wymagania dla bazy danych:                                                           --
--  * uprawnienia do tworzenia tabel                                                               --
--  * uprawnienia do tworzenia widokw                                                             --
--  * uprawnienia do tworzenia i uruchamiania wyzwalaczy                                           --
-- Kontrolnie tworzone s tabele/widoki z przedrostkiem patton_, pozwalajce w razie czego ogarn,--
-- co si dzieje.                                                                                  --
-- ----------------------------------------------------------------------------------------------- --

-- Widok trzymajcy rozdania, w ktrych s ju oba zapisy - tylko te rozdania s dalej brane pod uwag.
DROP VIEW IF EXISTS patton_boards;
CREATE VIEW patton_boards AS
	SELECT rnd, segment, tabl, board FROM scores WHERE score IS NOT NULL GROUP BY rnd, segment, tabl, board HAVING COUNT(*) = 2;

-- Tabela kompilujca wyniki rozda, zestawiajca zapisy z obu stow oraz wyrwnania wzgldem BAMw (dla rozda, gdzie pada remis +/- 10)
DROP TABLE IF EXISTS patton_scores;
CREATE TABLE patton_scores (
	rnd INT, -- z bazy Teamw
	segment INT, -- z bazy Teamw
	tabl INT, -- z bazy Teamw
	board INT, -- z bazy Teamw
	open_score INT, -- score ze scores dla pokoju otwartego
	closed_score INT, -- score ze scores dla pokoju zamknitego
	h_bam FLOAT, -- punkty BAMowe dla gospodarzy
	v_bam FLOAT -- punkty BAMowe dla goci
);
	
-- Tabela kompilujca saldo druzyn w meczu
DROP TABLE IF EXISTS patton_sums;
CREATE TABLE patton_sums (
	rnd INT, -- z bazy Teamw
	segment INT, -- z bazy Teamw
	tabl INT, -- z bazy Teamw
	h_saldo INT, -- saldo gospodarzy
	v_saldo INT, -- saldo goci
	max_saldo INT, -- wiksza z 2 powyszych wartoci (maksymalne saldo)
	h_points FLOAT, -- punkty za saldo dla gospodarzy
	v_points FLOAT -- punkty za saldo dla goci
);

-- Tabela kompilujca wyrwnania (takie, by wynik meczu w VP by rwny wynikowi wynikajcemu z Pattona)
DROP TABLE IF EXISTS patton_adjustments;
CREATE TABLE patton_adjustments (
	rnd INT, -- z bazy Teamw
	segment INT, -- z bazy Teamw
	tabl INT, -- z bazy Teamw
	h_total FLOAT, -- wyrwnanie dla gospodarzy
	v_total FLOAT -- wyrwnanie dla goci
);

-- Tabela zapamitujca wszelkie "rczne" zmiany na kolumnach corrv i corrh tabeli matches - wic zwyke turniejowe wyrwnania.
-- Zapamitujemy celem naoenia na wyrwnania wynikajce z wyniku meczu Pattonem.
CREATE TABLE IF NOT EXISTS patton_external_adjustments (
	rnd INT, -- z bazy Teamw
	tabl INT, -- z bazy Teamw
	h_adj FLOAT, -- wyrwnanie dla gospodarzy
	v_adj FLOAT, -- wyrwnanie dla goci
	PRIMARY KEY (rnd, tabl)
);

SET @h_adj = 0;
SET @v_adj = 0;

DROP TRIGGER IF EXISTS patton_trigger_adjustment;
DELIMITER //
-- Wyzwalacz zapamitrujcy wszelkie "zewntrzne" zmiany na tabeli matches, w kolumnach corrv i corrh - a wic wyrwnania.
CREATE TRIGGER patton_trigger_adjustment BEFORE UPDATE ON matches FOR EACH ROW BEGIN
	IF @patton_disable_trigger <> 1 OR @patton_disable_trigger IS NULL THEN
		SET @h_adj = NEW.corrh - COALESCE(OLD.corrh, 0);
		SET @v_adj = NEW.corrv - COALESCE(OLD.corrv, 0);
		-- Zapamitujemy do patton_external_adjustements, wstawiajc rekordy, jeli trzeba.
		IF (SELECT COUNT(*) FROM patton_external_adjustments WHERE rnd = NEW.rnd AND tabl = NEW.tabl) THEN
			UPDATE patton_external_adjustments
				SET h_adj = h_adj + @h_adj, v_adj = v_adj + @v_adj
				WHERE rnd = NEW.rnd AND tabl = NEW.tabl;
		ELSE
			INSERT INTO patton_external_adjustments(rnd, tabl, h_adj, v_adj)
				VALUES(NEW.rnd, NEW.tabl, @h_adj, @v_adj);
		END IF;
		SET @h_adj = 0;
		SET @v_adj = 0;
	END IF;
END //
DELIMITER ;

-- Na czas wykonywania skryptu wyczamy powyszy wyzwalacz (skrypt rwniez edytuje matches, w kocu)
SET @patton_disable_trigger = 1;

-- Kompilujemy tabel wynikw rozda i wyrwniania dla czi BAM z rozda.
DELETE FROM patton_scores;
INSERT INTO patton_scores
	SELECT pb.*,
		s1.score AS open_score,
		s2.score AS closed_score,
		-- Rodzania z rnic +/- 10 wymagaj wyrwnania -1 na niekorzy strony, ktra wzia +10
		IF(ABS(s1.score - s2.score) = 10, IF(s1.score > s2.score, -1, 1), 0) AS h_bam,
		0 AS v_bam
			FROM patton_boards pb
			JOIN scores s1 ON pb.rnd = s1.rnd AND pb.segment = s1.segment AND pb.tabl = s1.tabl AND pb.board = s1.board AND s1.room = 1
			JOIN scores s2 ON pb.rnd = s2.rnd AND pb.segment = s2.segment AND pb.tabl = s2.tabl AND pb.board = s2.board AND s2.room = 2;
UPDATE patton_scores SET v_bam = -h_bam;

-- Zmienna pomocnicza do wyliczenia punktw za saldo w zalenoci od liczby rozda w rundzie.
SET @boards_per_segment = IF ((SELECT boardspersegment FROM admin) = 4, 1, 0.5); -- Dla 4 rozda: = 1.0, dla 3 rozda: = 0.5

-- Wypeniamy tabel salda.
DELETE FROM patton_sums;
INSERT INTO patton_sums
	SELECT
		patton_scores.rnd, patton_scores.segment, patton_scores.tabl,
		SUM(IF(open_score > 0, open_score, 0)) + SUM(IF(closed_score < 0, -closed_score, 0)) AS h_saldo,
		SUM(IF(open_score < 0, -open_score, 0)) + SUM(IF(closed_score > 0, closed_score, 0)) AS v_saldo,
		0 AS max_saldo,
		0.0 AS h_points,
		0.0 AS v_points
		FROM patton_scores
		GROUP BY rnd, segment, tabl;

-- Wybieramy maksymalne saldo
UPDATE patton_sums SET max_saldo = IF (h_saldo > v_saldo, h_saldo, v_saldo);

-- Roboczo liczymy wynik za saldo wzgldem wyniku 0:0
-- Jeli rznica salda > 1/3 maksymalnego, to gospodarze zdobywaj:
--  * 2 punkty przy 4 rozdaniach
--  * 1 punkt przy 3 rozdaniach
UPDATE patton_sums SET h_points = 2 * @boards_per_segment
	WHERE (max_saldo - v_saldo) / max_saldo > 1/3;
-- Jeli rznica salda > 1/10 maksymalnego, ale < 1/3, to gospodarze zdobywaj:
--  * 2 punkty przy 4 rozdaniach
--  * 1 punkt przy 3 rozdaniach
UPDATE patton_sums SET h_points = 1 * @boards_per_segment
	WHERE (max_saldo - v_saldo) / max_saldo BETWEEN 1/10 AND 1/3;
-- Jeli rznica salda > 1/10 maksymalnego, ale < 1/3, to gocie zdobywaj:
--  * 2 punkty przy 4 rozdaniach
--  * 1 punkt przy 3 rozdaniach
UPDATE patton_sums SET h_points = -2 * @boards_per_segment
	WHERE (max_saldo - h_saldo) / max_saldo > 1/3;
-- Jeli rznica salda > 1/10 maksymalnego, ale < 1/3, to gocie zdobywaj:
--  * 1 punkt przy 4 rozdaniach
--  * 0.5 punktu przy 3 rozdaniach
UPDATE patton_sums SET h_points = -1 * @boards_per_segment
	WHERE (max_saldo - h_saldo) / max_saldo BETWEEN 1/10 AND 1/3;
-- Druga druyna zdobywa dopenienie do zera.
UPDATE patton_sums SET v_points = -h_points;
-- Podnosimy wynik za saldo z punktu odniesienia 0:0 do waciwego remisu, zalenego od liczby rozda (2:2 dla 4, 1:1 dla 3)
UPDATE patton_sums SET v_points = v_points + 2 * @boards_per_segment;
UPDATE patton_sums SET h_points = h_points + 2 * @boards_per_segment;

-- Kompilujemy wyrwnania Pattonowe, jako sum wyrwna z BAMw i punktw za saldo
DELETE FROM patton_adjustments;
INSERT INTO patton_adjustments
	SELECT patton_sums.rnd, patton_sums.segment, patton_sums.tabl,
		SUM(patton_scores.h_bam) + patton_sums.h_points AS h_total,
		SUM(patton_scores.v_bam) + patton_sums.v_points AS v_total
		FROM patton_sums
		JOIN patton_scores ON patton_sums.rnd = patton_scores.rnd AND patton_sums.segment = patton_scores.segment AND patton_sums.tabl = patton_scores.tabl
		GROUP BY patton_scores.rnd, patton_scores.segment, patton_scores.tabl;

-- Ustawiamy wyrwnania w matches, nanoszc na nie zapamitane "zewntrzne" wyrwnania
UPDATE matches SET
	corrh = COALESCE((SELECT h_total FROM patton_adjustments WHERE matches.rnd = patton_adjustments.rnd AND matches.tabl = patton_adjustments.tabl AND patton_adjustments.segment = 1), 0)
		+ COALESCE((SELECT h_adj FROM patton_external_adjustments WHERE matches.rnd = patton_external_adjustments.rnd AND matches.tabl = patton_external_adjustments.tabl), 0),
	corrv = COALESCE((SELECT v_total FROM patton_adjustments WHERE matches.rnd = patton_adjustments.rnd AND matches.tabl = patton_adjustments.tabl AND patton_adjustments.segment = 1), 0)
		+ COALESCE((SELECT v_adj FROM patton_external_adjustments WHERE matches.rnd = patton_external_adjustments.rnd AND matches.tabl = patton_external_adjustments.tabl), 0);

-- Oblokowujemy obsug wyzwalacza, na wypadek gdybymy chcieli co jeszcze robi na tym samym poczeniu do bazy.
SET @patton_disable_trigger = 0;
